
import { __decorate, __param } from '@codingame/monaco-vscode-api/external/tslib/tslib.es6';
import { localize, localize2 } from '@codingame/monaco-vscode-api/vscode/vs/nls';
import { Disposable } from '@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle';
import { Registry } from '@codingame/monaco-vscode-api/vscode/vs/platform/registry/common/platform';
import { LifecyclePhase } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/lifecycle/common/lifecycle';
import { registerAction2, MenuRegistry, MenuId } from '@codingame/monaco-vscode-api/vscode/vs/platform/actions/common/actions';
import { ProblemMatcherRegistry } from '../common/problemMatcher.js';
import { ProgressLocation } from '@codingame/monaco-vscode-api/vscode/vs/platform/progress/common/progress';
import { IProgressService } from '@codingame/monaco-vscode-api/vscode/vs/platform/progress/common/progress.service';
import { Extensions as Extensions$3 } from '@codingame/monaco-vscode-api/vscode/vs/platform/jsonschemas/common/jsonContributionRegistry';
import { StatusbarAlignment } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/statusbar/browser/statusbar';
import { IStatusbarService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/statusbar/browser/statusbar.service';
import { Extensions as Extensions$1 } from '@codingame/monaco-vscode-api/vscode/vs/workbench/services/output/common/output';
import { TASKS_CATEGORY, TaskEventKind, TaskGroup, TASK_RUNNING_STATE, TaskSettingId } from '@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/tasks/common/tasks';
import { TaskExecutionSupportedContext, TaskCommandsRegistered } from '@codingame/monaco-vscode-625898ab-0e33-5b7f-a3ae-29c10271dc1c-common/vscode/vs/workbench/contrib/tasks/common/taskService';
import { ITaskService } from '@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/tasks/common/taskService.service';
import { Extensions, registerWorkbenchContribution2, WorkbenchPhase } from '@codingame/monaco-vscode-api/vscode/vs/workbench/common/contributions';
import { RunAutomaticTasks, ManageAutomaticTaskRunning } from './runAutomaticTasks.js';
import { KeybindingsRegistry, KeybindingWeight } from '@codingame/monaco-vscode-api/vscode/vs/platform/keybinding/common/keybindingsRegistry';
import { KeyMod, KeyCode } from '@codingame/monaco-vscode-api/vscode/vs/base/common/keyCodes';
import schema$1 from '../common/jsonSchema_v1.js';
import schema$2, { updateProblemMatchers, updateTaskDefinitions } from '../common/jsonSchema_v2.js';
import { ConfigureTaskAction, AbstractTaskService } from './abstractTaskService.js';
import { tasksSchemaId } from '@codingame/monaco-vscode-37e80bf5-92f5-5e4c-8b6d-98e0bb89dbef-common/vscode/vs/workbench/services/configuration/common/configuration';
import { Extensions as Extensions$4 } from '@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configurationRegistry';
import { WorkbenchStateContext } from '@codingame/monaco-vscode-api/vscode/vs/workbench/common/contextkeys';
import { Extensions as Extensions$2 } from '@codingame/monaco-vscode-api/vscode/vs/platform/quickinput/common/quickAccess';
import { TasksQuickAccessProvider } from './tasksQuickAccess.js';
import { ContextKeyExpr } from '@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey';
import { TaskDefinitionRegistry } from '@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/tasks/common/taskDefinitionRegistry';
import { TerminalMenuBarGroup } from '@codingame/monaco-vscode-625898ab-0e33-5b7f-a3ae-29c10271dc1c-common/vscode/vs/workbench/contrib/terminal/browser/terminalMenus';
import { isString } from '@codingame/monaco-vscode-api/vscode/vs/base/common/types';
import { promiseWithResolvers } from '@codingame/monaco-vscode-api/vscode/vs/base/common/async';

const workbenchRegistry = ( Registry.as(Extensions.Workbench));
workbenchRegistry.registerWorkbenchContribution(RunAutomaticTasks, LifecyclePhase.Eventually);
registerAction2(ManageAutomaticTaskRunning);
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: ManageAutomaticTaskRunning.ID,
        title: ManageAutomaticTaskRunning.LABEL,
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
let TaskStatusBarContributions = class TaskStatusBarContributions extends Disposable {
    constructor(_taskService, _statusbarService, _progressService) {
        super();
        this._taskService = _taskService;
        this._statusbarService = _statusbarService;
        this._progressService = _progressService;
        this._activeTasksCount = 0;
        this._registerListeners();
    }
    _registerListeners() {
        let promise = undefined;
        let resolve;
        this._register(this._taskService.onDidStateChange(event => {
            if (event.kind === TaskEventKind.Changed) {
                this._updateRunningTasksStatus();
            }
            if (!this._ignoreEventForUpdateRunningTasksCount(event)) {
                switch (event.kind) {
                    case TaskEventKind.Active:
                        this._activeTasksCount++;
                        if (this._activeTasksCount === 1) {
                            if (!promise) {
                                ({ promise, resolve } = promiseWithResolvers());
                            }
                        }
                        break;
                    case TaskEventKind.Inactive:
                        if (this._activeTasksCount > 0) {
                            this._activeTasksCount--;
                            if (this._activeTasksCount === 0) {
                                if (promise && resolve) {
                                    resolve();
                                }
                            }
                        }
                        break;
                    case TaskEventKind.Terminated:
                        if (this._activeTasksCount !== 0) {
                            this._activeTasksCount = 0;
                            if (promise && resolve) {
                                resolve();
                            }
                        }
                        break;
                }
            }
            if (promise && (event.kind === TaskEventKind.Active) && (this._activeTasksCount === 1)) {
                this._progressService.withProgress({ location: ProgressLocation.Window, command: 'workbench.action.tasks.showTasks' }, progress => {
                    progress.report({ message: ( localize(9067, 'Building...')) });
                    return promise;
                }).then(() => {
                    promise = undefined;
                });
            }
        }));
    }
    async _updateRunningTasksStatus() {
        const tasks = await this._taskService.getActiveTasks();
        if (tasks.length === 0) {
            if (this._runningTasksStatusItem) {
                this._runningTasksStatusItem.dispose();
                this._runningTasksStatusItem = undefined;
            }
        }
        else {
            const itemProps = {
                name: ( localize(9068, "Running Tasks")),
                text: `$(tools) ${tasks.length}`,
                ariaLabel: ( localize(9069, "{0} running tasks", tasks.length)),
                tooltip: ( localize(9070, "Show Running Tasks")),
                command: 'workbench.action.tasks.showTasks',
            };
            if (!this._runningTasksStatusItem) {
                this._runningTasksStatusItem = this._statusbarService.addEntry(itemProps, 'status.runningTasks', StatusbarAlignment.LEFT, 49 );
            }
            else {
                this._runningTasksStatusItem.update(itemProps);
            }
        }
    }
    _ignoreEventForUpdateRunningTasksCount(event) {
        if (!this._taskService.inTerminal() || event.kind === TaskEventKind.Changed) {
            return false;
        }
        if ((isString(event.group) ? event.group : event.group?._id) !== TaskGroup.Build._id) {
            return true;
        }
        return event.__task.configurationProperties.problemMatchers === undefined || event.__task.configurationProperties.problemMatchers.length === 0;
    }
};
TaskStatusBarContributions = ( __decorate([
    ( __param(0, ITaskService)),
    ( __param(1, IStatusbarService)),
    ( __param(2, IProgressService))
], TaskStatusBarContributions));
workbenchRegistry.registerWorkbenchContribution(TaskStatusBarContributions, LifecyclePhase.Restored);
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: TerminalMenuBarGroup.Run,
    command: {
        id: 'workbench.action.tasks.runTask',
        title: ( localize(9071, "&&Run Task..."))
    },
    order: 1,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: TerminalMenuBarGroup.Run,
    command: {
        id: 'workbench.action.tasks.build',
        title: ( localize(9072, "Run &&Build Task..."))
    },
    order: 2,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: TerminalMenuBarGroup.Manage,
    command: {
        precondition: TASK_RUNNING_STATE,
        id: 'workbench.action.tasks.showTasks',
        title: ( localize(9073, "Show Runnin&&g Tasks..."))
    },
    order: 1,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: TerminalMenuBarGroup.Manage,
    command: {
        precondition: TASK_RUNNING_STATE,
        id: 'workbench.action.tasks.restartTask',
        title: ( localize(9074, "R&&estart Running Task..."))
    },
    order: 2,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: TerminalMenuBarGroup.Manage,
    command: {
        precondition: TASK_RUNNING_STATE,
        id: 'workbench.action.tasks.terminate',
        title: ( localize(9075, "&&Terminate Task..."))
    },
    order: 3,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: TerminalMenuBarGroup.Configure,
    command: {
        id: 'workbench.action.tasks.configureTaskRunner',
        title: ( localize(9076, "&&Configure Tasks..."))
    },
    order: 1,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.MenubarTerminalMenu, {
    group: TerminalMenuBarGroup.Configure,
    command: {
        id: 'workbench.action.tasks.configureDefaultBuildTask',
        title: ( localize(9077, "Configure De&&fault Build Task..."))
    },
    order: 2,
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.openWorkspaceFileTasks',
        title: ( localize2(9078, "Open Workspace Tasks")),
        category: TASKS_CATEGORY
    },
    when: ( ContextKeyExpr.and(( WorkbenchStateContext.isEqualTo('workspace')), TaskExecutionSupportedContext))
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: ConfigureTaskAction.ID,
        title: ConfigureTaskAction.TEXT,
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.showLog',
        title: ( localize2(9079, "Show Task Log")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.runTask',
        title: ( localize2(9080, "Run Task")),
        category: TASKS_CATEGORY
    }
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.reRunTask',
        title: ( localize2(9081, "Rerun Last Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.restartTask',
        title: ( localize2(9082, "Restart Running Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.showTasks',
        title: ( localize2(9083, "Show Running Tasks")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.terminate',
        title: ( localize2(9084, "Terminate Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.build',
        title: ( localize2(9085, "Run Build Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.test',
        title: ( localize2(9086, "Run Test Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.configureDefaultBuildTask',
        title: ( localize2(9087, "Configure Default Build Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.configureDefaultTestTask',
        title: ( localize2(9088, "Configure Default Test Task")),
        category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
MenuRegistry.appendMenuItem(MenuId.CommandPalette, {
    command: {
        id: 'workbench.action.tasks.openUserTasks',
        title: ( localize2(9089, "Open User Tasks")), category: TASKS_CATEGORY
    },
    when: TaskExecutionSupportedContext
});
class UserTasksGlobalActionContribution extends Disposable {
    constructor() {
        super();
        this.registerActions();
    }
    registerActions() {
        const id = 'workbench.action.tasks.openUserTasks';
        const title = ( localize(9090, "Tasks"));
        this._register(MenuRegistry.appendMenuItem(MenuId.GlobalActivity, {
            command: {
                id,
                title
            },
            when: TaskExecutionSupportedContext,
            group: '2_configuration',
            order: 6
        }));
        this._register(MenuRegistry.appendMenuItem(MenuId.MenubarPreferencesMenu, {
            command: {
                id,
                title
            },
            when: TaskExecutionSupportedContext,
            group: '2_configuration',
            order: 6
        }));
    }
}
workbenchRegistry.registerWorkbenchContribution(UserTasksGlobalActionContribution, LifecyclePhase.Restored);
KeybindingsRegistry.registerKeybindingRule({
    id: 'workbench.action.tasks.build',
    weight: KeybindingWeight.WorkbenchContrib,
    when: TaskCommandsRegistered,
    primary: KeyMod.CtrlCmd | KeyMod.Shift | KeyCode.KeyB
});
const outputChannelRegistry = ( Registry.as(Extensions$1.OutputChannels));
outputChannelRegistry.registerChannel({ id: AbstractTaskService.OutputChannelId, label: AbstractTaskService.OutputChannelLabel, log: false });
const quickAccessRegistry = (( Registry.as(Extensions$2.Quickaccess)));
const tasksPickerContextKey = 'inTasksPicker';
quickAccessRegistry.registerQuickAccessProvider({
    ctor: TasksQuickAccessProvider,
    prefix: TasksQuickAccessProvider.PREFIX,
    contextKey: tasksPickerContextKey,
    placeholder: ( localize(9091, "Type the name of a task to run.")),
    helpEntries: [{ description: ( localize(9092, "Run Task")), commandCenterOrder: 60 }]
});
const schema = {
    id: tasksSchemaId,
    description: 'Task definition file',
    type: 'object',
    allowTrailingCommas: true,
    allowComments: true,
    default: {
        version: '2.0.0',
        tasks: [
            {
                label: 'My Task',
                command: 'echo hello',
                type: 'shell',
                args: [],
                problemMatcher: ['$tsc'],
                presentation: {
                    reveal: 'always'
                },
                group: 'build'
            }
        ]
    }
};
schema.definitions = {
    ...schema$1.definitions,
    ...schema$2.definitions,
};
schema.oneOf = [...(schema$2.oneOf || []), ...(schema$1.oneOf || [])];
const jsonRegistry = ( Registry.as(Extensions$3.JSONContribution));
jsonRegistry.registerSchema(tasksSchemaId, schema);
class TaskRegistryContribution extends Disposable {
    static { this.ID = 'taskRegistryContribution'; }
    constructor() {
        super();
        this._register(ProblemMatcherRegistry.onMatcherChanged(() => {
            updateProblemMatchers();
            jsonRegistry.notifySchemaChanged(tasksSchemaId);
        }));
        this._register(TaskDefinitionRegistry.onDefinitionsChanged(() => {
            updateTaskDefinitions();
            jsonRegistry.notifySchemaChanged(tasksSchemaId);
        }));
    }
}
registerWorkbenchContribution2(TaskRegistryContribution.ID, TaskRegistryContribution, WorkbenchPhase.AfterRestored);
const configurationRegistry = ( Registry.as(Extensions$4.Configuration));
configurationRegistry.registerConfiguration({
    id: 'task',
    order: 100,
    title: ( localize(9093, "Tasks")),
    type: 'object',
    properties: {
        [TaskSettingId.ProblemMatchersNeverPrompt]: {
            markdownDescription: ( localize(
                9094,
                "Configures whether to show the problem matcher prompt when running a task. Set to `true` to never prompt, or use a dictionary of task types to turn off prompting only for specific task types."
            )),
            'oneOf': [
                {
                    type: 'boolean',
                    markdownDescription: ( localize(9095, 'Sets problem matcher prompting behavior for all tasks.'))
                },
                {
                    type: 'object',
                    patternProperties: {
                        '.*': {
                            type: 'boolean'
                        }
                    },
                    markdownDescription: ( localize(
                        9096,
                        'An object containing task type-boolean pairs to never prompt for problem matchers on.'
                    )),
                    default: {
                        'shell': true
                    }
                }
            ],
            default: false
        },
        [TaskSettingId.AutoDetect]: {
            markdownDescription: ( localize(
                9097,
                "Controls enablement of `provideTasks` for all task provider extension. If the Tasks: Run Task command is slow, disabling auto detect for task providers may help. Individual extensions may also provide settings that disable auto detection."
            )),
            type: 'string',
            enum: ['on', 'off'],
            default: 'on'
        },
        [TaskSettingId.SlowProviderWarning]: {
            markdownDescription: ( localize(9098, "Configures whether a warning is shown when a provider is slow")),
            'oneOf': [
                {
                    type: 'boolean',
                    markdownDescription: ( localize(9099, 'Sets the slow provider warning for all tasks.'))
                },
                {
                    type: 'array',
                    items: {
                        type: 'string',
                        markdownDescription: ( localize(9100, 'An array of task types to never show the slow provider warning.'))
                    }
                }
            ],
            default: true
        },
        [TaskSettingId.QuickOpenHistory]: {
            markdownDescription: ( localize(
                9101,
                "Controls the number of recent items tracked in task quick open dialog."
            )),
            type: 'number',
            default: 30, minimum: 0, maximum: 30
        },
        [TaskSettingId.QuickOpenDetail]: {
            markdownDescription: ( localize(
                9102,
                "Controls whether to show the task detail for tasks that have a detail in task quick picks, such as Run Task."
            )),
            type: 'boolean',
            default: true
        },
        [TaskSettingId.QuickOpenSkip]: {
            type: 'boolean',
            description: ( localize(
                9103,
                "Controls whether the task quick pick is skipped when there is only one task to pick from."
            )),
            default: false
        },
        [TaskSettingId.QuickOpenShowAll]: {
            type: 'boolean',
            description: ( localize(
                9104,
                "Causes the Tasks: Run Task command to use the slower \"show all\" behavior instead of the faster two level picker where tasks are grouped by provider."
            )),
            default: false
        },
        [TaskSettingId.AllowAutomaticTasks]: {
            type: 'string',
            enum: ['on', 'off'],
            enumDescriptions: [
                ( localize(9105, "Always")),
                ( localize(9106, "Never")),
            ],
            description: ( localize(
                9107,
                "Enable automatic tasks - note that tasks won't run in an untrusted workspace."
            )),
            default: 'on',
            restricted: true
        },
        [TaskSettingId.Reconnection]: {
            type: 'boolean',
            description: ( localize(9108, "On window reload, reconnect to tasks that have problem matchers.")),
            default: true
        },
        [TaskSettingId.SaveBeforeRun]: {
            markdownDescription: ( localize(9109, 'Save all dirty editors before running a task.')),
            type: 'string',
            enum: ['always', 'never', 'prompt'],
            enumDescriptions: [
                ( localize(9110, 'Always saves all editors before running.')),
                ( localize(9111, 'Never saves editors before running.')),
                ( localize(9112, 'Prompts whether to save editors before running.')),
            ],
            default: 'always',
        },
        [TaskSettingId.VerboseLogging]: {
            type: 'boolean',
            description: ( localize(9113, "Enable verbose logging for tasks.")),
            default: false
        },
    }
});

export { TaskRegistryContribution, TaskStatusBarContributions };
